=comment

  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer
 
  $Id: Util.pm,v 1.12 2006/04/29 03:55:16 taku-ku Exp $;

  Copyright (C) 2000-2004 Taku Kudo <taku@chasen.org>
  Copyright (C) 2005      NTT Communication Science Laboratories
  This is free software with ABSOLUTELY NO WARRANTY.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
=cut

use strict;
use warnings;
use IO::File;
use File::Spec;
use FindBin;

BEGIN {
  if ($] >= 5.008) {
      use Encode qw (from_to);
      use Encode::Guess;
  } else {
      warn "Encode::from_to/Guess doesn't work on your computor\n";
  }
}

package Conf;
{
    our $TOOL_DIR   = $FindBin::Bin;
    our $MECAB_TOOL = File::Spec->catfile ($TOOL_DIR, "mecab-tool");
    if ($^O eq 'MSWin32') {
	$MECAB_TOOL = File::Spec->catfile ($TOOL_DIR, "mecab-tool.exe");
	unless (-f $MECAB_TOOL) {
	    $MECAB_TOOL = File::Spec->catfile ($TOOL_DIR, "..\\bin\\mecab-tool.exe");
	}
    }
    
    if (defined $ENV{"MECAB_TOOL"}) {
	$MECAB_TOOL = $ENV{"MECAB_TOOL"};
    }
    
    our $MODEL_VERSION = 100;

    our $UNK_TXT_FILE  = "unk.def";
    our $UNK_DIC_FILE  = "unk.dic";
    our $SYS_DIC_FILE  = "sys.dic";

    # character map
    our $CHARCATEGORY_TXT_FILE = "char.def";
    our $CHARCATEGORY_BIN_FILE = "char.bin";

    # connection matrix
    our $MATRIX_TXT_FILE = "matrix.def";
    our $MATRIX_BIN_FILE = "matrix.bin";

    # FEATURE -> Internal feature map
    our $REWRITE_FILE  = "rewrite.def";
    
    # Connection ID
    our $LEFT_ID_FILE  = "left-id.def";
    our $RIGHT_ID_FILE = "right-id.def";

    # Feature templates
    our $FEATURE_FILE = "feature.def";

    # dicrc
    our $DICRC = "dicrc";

    # DIR
    our $DIC_DIR = ".";
    our $OUT_DIR = ".";
    
    our $DEFAULT_CHARSET = "euc-jp";
}

package Util;
{
    sub read_config {

	my $fh = Util::open_with_dicdir($Conf::DICRC);
	my %config;
	while (my $line = <$fh>) {
	    next if ($line =~ /^\#/ || $line =~ /^\s*$/);
	    if ($line =~ /([^=\s]+)\s*=\s*(\S+)/) {
		$config{$1} = $2;
	    }
	}
	close ($fh);
	return %config;
    }

    sub catfiles {

	my $ofile = shift @_;
	my $fh = Util::wopen ($ofile);
	binmode $fh;
	for my $file (@_) {
	    my $buf;
	    open (F, $file) || die "$!: $file\n"; 
	    binmode F;
	    while (1) {
		my $ret = read (F, $buf, 8192);
		print $fh $buf;
		last if ($ret < 8192);
	    }
	    close (F);
	}
	close ($fh);
    }
    
    sub set_config($$) {
       my ($dic, $out) = @_;
       $Conf::DIC_DIR = $dic if (defined $dic);
       $Conf::OUT_DIR = $out if (defined $out);
    }

    sub enum_csv_dictionaries() {
	opendir (DICDIR, $Conf::DIC_DIR) || die "$! $Conf::DIC_DIR\n";
	my @dic = ();
	for (grep (/\.csv$/i, readdir (DICDIR))) {
	    push @dic, File::Spec->catfile ($Conf::DIC_DIR, $_);
	}
	return wantarray ? @dic : \@dic;
    }
    
    sub splitCSV {
        my $line = shift;
        my $size = shift || -1;;
        my @values = ();
        while ($line =~ s/("[^"]*(?:""[^"]*)*"|[^,]*),?//) {
	      my $tmp = $1;
              $tmp =~ s/^"(.*)"$/$1/ && $tmp =~ s/""/"/g; # "
              push (@values, $tmp);
	      last if (! defined $line || $line eq "");
              if (@values == $size - 1) {
		  push @values, $line;
		  last;
              }
        }							      
	return wantarray ? @values : \@values;
    }

    sub toCSV {
        my $field = shift;
	return join ',', map {(s/"/""/g or /[\r\n,]/) ? qq("$_") : $_} @{$field}; # "
    }

    sub open {
	my $file = shift @_;
        my $fh = new IO::File;
        $fh->open ($file) || die "$! $file\n";
	return $fh;
    }

    sub wopen {
	my $file = shift @_;    
        my $fh = new IO::File;
        $fh->open ("> $file") || die "$! $file\n";
	return $fh;
    }
    
    sub open_with_dicdir {
	my $file = File::Spec->catfile ($Conf::DIC_DIR, shift @_);
        my $fh = new IO::File;
        $fh->open ($file) || die "$! $file\n";
	return $fh;
    }

    sub wopen_with_outdir {
	my $file = File::Spec->catfile ($Conf::OUT_DIR, shift @_);
        my $fh = new IO::File;
        $fh->open ("> $file") || die "$! $file\n";
	return $fh;
    }
    
    sub dicfile {
        return  File::Spec->catfile ($Conf::DIC_DIR, shift @_);
    }
    
    sub outfile {
        return  File::Spec->catfile ($Conf::OUT_DIR, shift @_);
    }
    
    sub tmpfile {
        my $file = "";
    	while (1) {
	    $file = File::Spec->catfile (File::Spec->tmpdir (), 
	                                 sprintf ("mecab-%d.%d", $$, int (rand(10000))));
	    last unless (-f $file);
        }
	return $file;
    }
	    
    sub code_conv {
	my ($str, $from, $to) = @_;
	return if ($from eq $to);
	Encode::from_to($$str, $from, $to) if ($] >= 5.008);
    }
	    
    sub guess_encoding {
	return $Conf::DEFAULT_CHARSET unless ($] >= 5.008);
	my $file = shift;    	
	my $fh   = Util::open($file);
	my $trial = 0;
	my $success = 0;
	my $str = "";
	my $result;
	my $n = 0;

	while (1) {

	    my $readall = 1;
	    
	    while (my $line = <$fh>) {
		$str .= $line;
		if ($n++ % 20 == 0) {
		    $readall = 0;
		    last;
		}
	    }

	    my $from = Encode::Guess::guess_encoding
	      ($str, qw/euc-jp sjis utf-8/);
	    
	    if (ref($from)) {
		$result = $from->name;
		$success = 1;
		last;
	    }
	    
	    last if ($readall || $trial++ == 20);
	}
    
	unless ($success) {
	    warn "Encode::Guess failed. use $Conf::DEFAULT_CHARSET\n";
	    $result = $Conf::DEFAULT_CHARSET;
	}
	
	close($fh);
	
	return $result;
    }
}

package CharacterCategory;
{
    sub new {

	my $this = {};

	my %category;
	my @category_list;
	my @map;


	my $fh = Util::open_with_dicdir ($Conf::CHARCATEGORY_TXT_FILE);
	my $id = 0;
	
	while (my $line = <$fh>) {

	    chomp $line;

	    next if ($line =~ /^\s*\#/ || $line =~ /^\s*$/);

	    $line =~ s/\#.+$//; # remove comment
	    $line =~ s/\s+$//;  # remove spaces
	    
	    my $low  = -1;
	    my $high = -1;
	    my $cat;

	    if ($line =~ /^(0x[0-9A-Fa-f]{2,4})\.\.(0x[0-9A-Fa-f]{2,4})\s+(.+)/) {

		$low  = hex($1);
		$high = hex($2);
		$cat  = [ split /\s+/, $3 ];

	    } elsif ($line =~ /^(0x[0-9A-Fa-f]{2,4})\s+(.+)/) {

		$low  = hex($1);
		$high = hex($1);
		$cat  = [ split /\s+/, $2 ];

	    } elsif ($line =~ /^(\S+)\s+([01])\s+([01])\s+(\d+)/) {

		die "length parameter is too large: $4\n" if ($4 > 16);
		
		if (! defined $category{$1}) {
		    $category{$1} = { invoke => $2, group => $3, length => $4, id => $id };
		    ++$id;
		} else {
		    die "category $1 is already defined\n";
		}

	    } else {
		
		die "format error: $line\n";
	    }

	    if ($low >= 0 && $high >= 0) {
		for my $c (@$cat) {
		    die "undefined category $c" unless $category{$c};
		}
		push @map, { low => $low, high => $high, cat => $cat };
	    }
	}

	close ($fh);

	die "too many categories (> 18) \n" if (scalar (keys %category) >= 18);
	die "DEFAULT category is undefined\n" unless ($category{"DEFAULT"});
	
	my %unk = ();
	$fh = Util::open_with_dicdir ($Conf::UNK_TXT_FILE);
	while (my $line = <$fh>) {
	    chomp $line;
	    next if ($line =~ /^\s*\#/ || $line =~ /^\s*$/ );
	    my ($w, $feature) = Util::splitCSV($line, 2);
	    die "category [$w] is not defined in $Conf::CHARCATEGORY_TXT_FILE\n" unless ($category{$w});
	    $unk{$w} = 1;
	}
	close ($fh);

	for my $u (keys %category) {
	    die "category [$u] is not defined in $Conf::UNK_TXT_FILE\n" unless ($unk{$u});
	}

	$this->{_category} = \%category;
	$this->{_map}      = \@map;
	
	bless ($this);
    }

    sub getID  {
	my $this = shift @_;
	my $cat  = shift @_;
	my $id   = $this->{_category}->{$cat}->{id};
	die "CharacterCategory::getID(): undefined category $cat\n" unless ($id);
	return $id;
    }

    sub size {
	my $this = shift @_;
	return scalar (keys %{$this->{_category}});
    }

    sub encode {
	my $this = shift @_;
	my $c    = shift @_;
	my $h    = $this->{_category}->{$c->[0]}; # default
	my $base = ($h->{invoke} << 31) + 
	           ($h->{group}  << 30) +
   		   ($h->{length} << 26) + 
		   ($h->{id}     << 18) ;
	for my $u (@$c) {
	    $base += (1 << $this->{_category}->{$u}->{id});
	}
	return $base;
    }

    sub save {
	my $this = shift @_;
	my $file = $Conf::CHARCATEGORY_BIN_FILE;

	my $category = $this->{_category};
	my $map      = $this->{_map};
	my @out;

	my $c = $this->encode ( [ "DEFAULT" ] );
	for (my $i = 0; $i < 0xffff; ++$i) {
	    push @out, $c;
	}	

	for my $m (@$map) {
	    my $low  = $m->{low};
	    my $high = $m->{high};
	    my $cat  = $m->{cat};
	    for (my $j = $low; $j <= $high; ++$j) {
		$out[$j] = $this->encode ($cat);
	    }
	}

	my $fh = Util::wopen_with_outdir ($file);
	binmode $fh;
	my @clist = sort { $category->{$a}->{id} <=> $category->{$b}->{id} } keys %$category;
	print $fh pack ("I", scalar @clist);
	for my $c (@clist) {
	    print $fh pack ("a32", $c);
	}
	print $fh pack ("I*", @out);
	close ($fh);
    }
}

package ConnectionMatrix;
{
    sub convert {
	my $text   = Util::dicfile ($Conf::MATRIX_TXT_FILE);
	my $binary = Util::outfile ($Conf::MATRIX_BIN_FILE);
	system ($Conf::MECAB_TOOL, "--convmatrix", "--input=$text", "--output=$binary");
    }

    sub new {
	my $this = {};
	$this->{_left_size} = 0;
	$this->{_right_size} = 0;
	bless ($this);
    }

    sub open {
	my $this = shift @_;
	my $fh = Util::open_with_dicdir ($Conf::MATRIX_TXT_FILE);
	my $line = <$fh>;
	chomp $line;
	my ($l, $r) = split /\s+/, $line, 2;
	$this->{_left_size} = $l;
	$this->{_right_size} = $r;
	close($fh);
    }

    sub left_size {
    	my $this = shift @_;
	return $this->{_left_size};
    }
    
    sub right_size {
    	my $this = shift @_;
	return $this->{_right_size};
    }

    sub is_valid {
    	my ($this, $lid, $rid) = @_;
	# Note that $lid < $this->{_right_size} &&
	# [lid right] in Token [lid1 rid1] -> [lid2 rid2]
	# connection is rid1 -> lid2
	return 1 if ($lid >= 0 && $lid < $this->{_right_size} &&
	    $rid >= 0 && $rid < $this->{_left_size} );
        return 0;
    }
}

package ContextID;
{
    sub new {
	my $this = shift;
	$this = {};
	$this->{_left}  = {};
	$this->{_right} = {};
	bless ($this);
    }

    sub open {
	my $this = shift @_;
	$this->{_left}       = ContextID::_open ($Conf::LEFT_ID_FILE);
	$this->{_right}      = ContextID::_open ($Conf::RIGHT_ID_FILE);
	$this->{_left_size}  = scalar keys %{$this->{_left}};
	$this->{_right_size} = scalar keys %{$this->{_right}};
    }

    sub add {
	my ($this, $l, $r)  = @_;
	$this->{_left}->{$l}  = 1;
	$this->{_right}->{$r} = 1;
    }
    
    sub addBOS {
	my ($this, $l, $r)  = @_;
	$this->{_left_bos}  = $l;
	$this->{_right_bos} = $r;
    }
    
    sub save {
	my $this = shift @_;
	ContextID::_save($Conf::LEFT_ID_FILE,  $this->{_left});
	ContextID::_save($Conf::RIGHT_ID_FILE, $this->{_right});
    }

    sub buildID {
	my $this = shift @_;
	ContextID::_buildID($this->{_left},  $this->{_left_bos});
	ContextID::_buildID($this->{_right}, $this->{_right_bos});
    }

    sub getID {
	my ($this, $l, $r) = @_;

	my $lid = $this->{_left}->{$l};
	my $rid = $this->{_right}->{$r};

	die "ContextID::getID(): cannot find LEFT-ID  for [$l]\n" unless (defined $lid) ;
	die "ContextID::getID(): cannot find RIGHT-ID for [$r]\n" unless (defined $rid) ;

	return ($lid, $rid);
    }
    
    sub left_size {
    	my $this = shift @_;
	return $this->{_left_size};
    }
    
    sub right_size {
    	my $this = shift @_;
	return $this->{_right_size};
    }
    
    sub is_valid {
    	my ($this, $lid, $rid) = @_;
	
	return 1 if ($lid >= 0 && $lid < $this->{_left_size} &&
		     $rid >= 0 && $rid < $this->{_right_size} );
        return 0;
    }

    sub left {
    	my $this = shift @_;
	return $this->{_left};
    }
    
    sub right {
    	my $this = shift @_;
	return $this->{_right};
    }

    sub _open {
	my $file = shift @_;
	my $fh = Util::open_with_dicdir ($file);
	my %result;
	while (my $line = <$fh>) {
	    chomp $line;
	    my ($id, $str) = split /\s+/, $line, 2 ;
	    $result{$str} = $id;
	}
	close ($fh);
	return \%result;
    }

    sub _buildID {
	my $hash = shift @_;	
	my $bos  = shift @_;
	my $id = 1; # 0 is reserved for BOS/EOS	
	for my $s (sort keys %{$hash}) {
	    $$hash{$s} = $id;
	    ++$id;
	}
	$$hash{$bos} = 0 if (defined $bos);
    }

    sub _save {
	my $file = shift @_;
	my $hash = shift @_;
	my $fh = Util::wopen_with_outdir ($file);
	for my $s (sort { $$hash{$a} <=> $$hash{$b} } keys %{$hash}) {
	    print $fh "$$hash{$s} $s\n";
	}
	close ($fh);
    }
};

package RewriteDictinary;
{
    sub new {
	my $this = shift;
	$this = {};
	my $fh = Util::open_with_dicdir($Conf::REWRITE_FILE);
	my $all = join "", <$fh>;
	close($fh);
	eval($all);
	init_rewrite ($Conf::DIC_DIR);
	bless ($this);
    }

    sub rewrite {
	my $this = shift;
	my $line = shift;
	my ($w, $lid, $rid, $cost, $feature) = Util::splitCSV($line, 5);
	my $f        = Util::splitCSV($feature);
	my $ufeature = Util::toCSV(unigram_rewrite ($f));
	my $lfeature = Util::toCSV(left_rewrite    ($f));
	my $rfeature = Util::toCSV(right_rewrite   ($f));
	return ($w, $lid, $rid, $cost, $feature, $ufeature, $lfeature, $rfeature);
    }
}

package ProgressBar;
{
    sub new {
	my ($this, $title, $total) = @_;
	$this = {};
	$this->{_title} = $title;
	$this->{_total} = $total;
	$this->{_prev}  = 0;
	$this->{_prevc} = 0;
	bless $this;
    }

    sub display {
	my ($this, $current) = @_;
	$current = $this->{_prevc} + 1 if (! defined $current);
	my $bar = "###########################################";
	my $scale = length ($bar) - 1;
	my $prev = $this->{_prev};
	my $total = $this->{_total};
	my $cur_percentage  = int(100.0 * $current/$total);
	my $bar_len         = int(1.0   * $current*$scale/$total);
	
	if ($prev != $cur_percentage) {
	    $| = 1;
	    printf "%s: %3d%% |%.*s%*s| ", 
	    $this->{_title}, $cur_percentage, $bar_len, $bar, $scale-$bar_len, "";
	    if ($cur_percentage == 100) { print STDERR "\n"; }
	    else                        { print STDERR "\r"; };
	}
	$this->{_prev}  = $cur_percentage;
	$this->{_prevc} = $current;
    }
}

1;
