/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: char_property.cpp,v 1.7 2006/04/25 17:16:21 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "mempool.h"
#include "common.h"
#include "char_property.h"
#include "strutil.h"

namespace MeCab
{
  bool CharProperty::open (Param &param)
  {
    std::string prefix   = param.getProfileString ("dicdir");
    std::string filename = createFileName (prefix, CHARACTER_CATEGORY_FILE);

    MMAP_OPEN(char, cmmap_, filename);

    char *ptr = cmmap_->begin();
    unsigned int csize;
    read_static<unsigned int>(&ptr, csize);

    size_t fsize = sizeof (unsigned int) +
      (32 * csize) + sizeof (unsigned int) * 0xffff;

    if (fsize != cmmap_->size())
      WHAT_CLOSE_FALSE("invalid file size: " << filename);

    clist_.clear ();
    for (unsigned int i = 0; i < csize; ++i) {
      char *s = read_ptr(&ptr, 32);
      clist_.push_back (s);
    }

    map_ = (CharInfo *)ptr;

    return true;
  }

  void CharProperty::close ()
  {
    MMAP_CLOSE(char, cmmap_);
  }

  size_t CharProperty::size ()  { return clist_.size(); }

  const char *CharProperty::name (size_t i)
  {
    return const_cast<const char*>(clist_[i]);
  }

  void CharProperty::set_charset (const char *ct)
  {
    // this function must be rewritten.
    std::string tmp = ct;
    if      (tmp == "sjis"  || tmp == "shift-jis" || tmp == "shift_jis" || tmp == "cp932") charset_ = CP932;
    else if (tmp == "euc"   || tmp == "euc_jp"    || tmp == "euc-jp")    charset_ = EUC_JP;
    else if (tmp == "utf8"  || tmp == "utf_8"     || tmp == "utf-8")     charset_ = UTF8;
    else if (tmp == "ascii") charset_ = ASCII; 
    else charset_ = UTF8; // default is UTF8
  }

  int CharProperty::id (const char *key)
  {
    for (int i = 0; i < (long)clist_.size(); ++i)
      if (strcmp (key, clist_[i]) == 0) return i;
    return -1;
  }
}
