/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: char_property.h,v 1.7 2006/04/25 17:16:21 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _MECAB_CHARACTER_CATEGORY_H
#define _MECAB_CHARACTER_CATEGORY_H

#include "param.h"
#include "mmap.h"
#include "ucs.h"
#include "string_buffer.h"

namespace MeCab
{
  struct CharInfo
  {
#ifdef WORDS_BIGENDIAN
    unsigned int invoke:       1;
    unsigned int group:        1;
    unsigned int length:       4;
    unsigned int default_type: 8;
    unsigned int type:         18;
#else
    unsigned int type:         18;
    unsigned int default_type: 8;
    unsigned int length:       4;
    unsigned int group:        1;
    unsigned int invoke:       1;
#endif
    bool isKindOf (CharInfo c) { return type & c.type; }
  };

  class CharProperty {

  private:

    enum { EUC_JP, CP932, UTF8, ASCII };

    MeCab::Mmap<char> *cmmap_;
    std::vector<char *> clist_;
    CharInfo *map_;
    int charset_;

    StringBuffer what_;

  public:

    CharProperty():  cmmap_(0), map_(0), charset_(0) {};
    virtual ~CharProperty() { this->close (); }

    bool   open (Param &);
    void   close ();
    size_t size ();
    const char *what () { return what_.str(); }

    void set_charset (const char *);
    int  id (const char *);
    const char *name (size_t i);

    inline char *seekToOtherType  (const char *begin, const char *end,
           CharInfo c, CharInfo &fail, size_t& mblen) const
    {
      register char *p = const_cast<char *>(begin);
      while (p != end && c.isKindOf (fail = getCharInfo (p, end, mblen))) {
	 p += mblen;
	 c = fail;
      }
      return p;
    }

    inline char *seekToOtherType (const char *begin, const char *end, CharInfo c) const
    {
      register size_t mblen;
      register char *p = const_cast<char *>(begin);
      register CharInfo c2; 
      while (p != end && c.isKindOf (c2 = getCharInfo (p, end, mblen))) {
	 p += mblen;
	 c = c2;
      }
      return p;
    }

    inline CharInfo getCharInfo (const char *begin, const char *end, size_t& mblen) const
    {
#ifndef MECAB_USE_UTF8_ONLY
      unsigned short int t = 0;
      switch (charset_) {
      case EUC_JP: t = euc_to_ucs2   (begin, end, mblen); break;
      case CP932:  t = cp932_to_ucs2 (begin, end, mblen); break;
      case UTF8:   t = utf8_to_ucs2  (begin, end, mblen); break;
      case ASCII:  t = ascii_to_ucs2 (begin, end, mblen); break;
      default:     t = utf8_to_ucs2  (begin, end, mblen); break; // default charcode is UTF-8
      }
#else
      unsigned short int t = utf8_to_ucs2  (begin, end, mblen);
#endif
      return map_[t];
    }

    inline CharInfo getCharInfo (size_t id) { return map_[id]; }
  };
}
#endif
