/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: common.h,v 1.3 2005/12/24 08:24:22 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _MECAB_COMMON_H
#define _MECAB_COMMON_H

#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <string>
#include <algorithm>
#include <cmath>

// tricky macro for MSVC
#ifdef _MSC_VER
#define for if(0); else for
/* why windows.h define such a generic macro */
#undef max
#undef min
#endif

#define COPYRIGHT "MeCab: Yet Another Part-of-Speech and Morphological Analyzer\n\
\nCopyright (C) 2001-2005 Taku Kudo \nCopyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation\n"

#define SYS_DIC_FILE             "sys.dic"
#define UNK_DIC_FILE             "unk.dic"
#define CONNECTION_FILE          "matrix.bin"
#define CHARACTER_CATEGORY_FILE  "char.bin"
#define FEATURE_DEF_FILE         "feature.def"

#define BOS_KEY "BOS/EOS"

#define NBEST_MAX (512)

#define NODE_FREELIST_SIZE (512)
#define PATH_FREELIST_SIZE (2048)

#define MIN_INPUT_BUFFER_SIZE  (8192)
#define MAX_INPUT_BUFFER_SIZE  (8192*640)

#ifndef EXIT_FAILURE
#define EXIT_FAILURE (1)
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS (0)
#endif

#define WHAT(msg) what_.clear(), what_ << __FILE__ << "(" << __LINE__ << "): " << msg;
#define WHAT_CLOSE_FALSE(msg) do { WHAT(msg); this->close(); return false; } while (0)
#define WHAT_FALSE(msg)       do { WHAT(msg); return false; } while (0)

namespace MeCab {

  template <class T> inline T _min (T x, T y) { return (x < y) ? x : y; }
  template <class T> inline T _max (T x, T y) { return (x > y) ? x : y; }

  inline double logsumexp (double x, double y, bool flg)
  {
#define MINUS_LOG_EPSILON  50

    if (flg) return y; // init mode
    double vmin = _min (x, y);
    double vmax = _max (x, y);
    if (vmax > vmin + MINUS_LOG_EPSILON) {
      return vmax;
    } else {
      return vmax + std::log (std::exp (vmin - vmax) + 1.0);
    }
  }
}
#endif
