/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: dictionary.cpp,v 1.6 2006/01/01 04:14:26 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "dictionary.h"
#include "common.h"
#include "strutil.h"
#include "mempool.h"

namespace MeCab {

  bool Dictionary::open (const std::string &file)
  {
    MMAP_OPEN(char, dmmap_, file);

    if (dmmap_->size() < 96)
      WHAT_CLOSE_FALSE("dictionary file is broken: " << file);

    char *ptr = dmmap_->begin();

    unsigned short index_type;
    unsigned int   dsize;
    unsigned int   tsize;
    unsigned int   fsize;
    unsigned int   lsize;
    unsigned int   rsize;

    read_static<unsigned short>(&ptr, version_);
    read_static<unsigned short>(&ptr, type_);
    read_static<unsigned short>(&ptr, learn_);
    read_static<unsigned short>(&ptr, index_type);
    read_static<unsigned int>  (&ptr, lexsize_);
    read_static<unsigned int>  (&ptr, lsize);
    read_static<unsigned int>  (&ptr, rsize);
    read_static<unsigned int>  (&ptr, dsize);
    read_static<unsigned int>  (&ptr, tsize);
    read_static<unsigned int>  (&ptr, fsize);

    charset_ = ptr;    ptr += 32;
    da_.set_array (ptr); ptr += dsize;
    token_ = reinterpret_cast<Token *>(ptr); ptr += tsize;
    feature_ = ptr;    ptr += fsize;

    if (version_ / 100 != (DIC_VERSION) / 100)
      WHAT_CLOSE_FALSE("incompatible version: " << version_/100);

    if (ptr != dmmap_->end())
      WHAT_CLOSE_FALSE("dictionary file is broken: " << file);

    return true;
  }

  void Dictionary::close ()
  {
    MMAP_CLOSE(char, dmmap_);
  }
}
