/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: dictionary.h,v 1.8 2006/04/23 14:38:31 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _MECAB_DICTIONARY_H
#define _MECAB_DICTIONARY_H

#include "mecab.h"
#include "darts.h"
#include "mmap.h"
#include "token.h"
#include "string_buffer.h"

namespace MeCab {

  class Dictionary {

  private:

    MeCab::Mmap<char>  *dmmap_;
    Token *token_;
    char  *feature_;
    char  *charset_;
    unsigned short version_;
    unsigned short type_;
    unsigned short learn_;
    unsigned int   lexsize_;
    unsigned int   lsize_;
    unsigned int   rsize_;
    StringBuffer  what_;
    Darts::DoubleArray da_;

  public:
    typedef Darts::DoubleArray::result_pair_type result_type;

    bool open (const std::string &);
    void close ();

    size_t commonPrefixSearch (const char* key, size_t len,
			       result_type *result,
			       size_t rlen) {
      return da_.commonPrefixSearch(key, result, rlen, len);
    }

    result_type exactMatchSearch (const char* key) {
      result_type n;
      da_.exactMatchSearch (key, n);
      return n;
    }

    bool isCompatible (const Dictionary &d) const
    {
      return (version_ == d.version_ &&
	      learn_   == d.learn_   &&
	      lsize_   == d.lsize_   &&
	      rsize_   == d.rsize_   &&
	      std::strcmp (charset_, d.charset_) == 0);
    }

    const char* charset () const { return const_cast<const char*>(charset_); }
    unsigned short version () const { return version_; }
    size_t  size () const { return static_cast<size_t>(lexsize_); }
    int learn () const { return static_cast<int>(learn_); }
    int type () const { return static_cast<int>(type_);  }
    size_t lsize () const { return static_cast<size_t>(lsize_); }
    size_t rsize () const { return static_cast<size_t>(rsize_); }


    Token *token(result_type &n)      const { return token_ + (n.value >> 8) ; }
    size_t token_size(result_type &n) const { return 0xff & n.value; };
    char  *feature (const Token &t) const { return feature_ + t.feature; }

    const char *what ()  { return what_.str(); }

    Dictionary(): dmmap_(0), token_(0), feature_(0), charset_(0) {};
    virtual ~Dictionary() { this->close (); }
  };
}
#endif
