/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: feature_index.h,v 1.3 2006/01/13 15:24:16 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef MECAB_FEATUREINDEX
#define MECAB_FEATUREINDEX

#include <map>
#include <vector>

#include "mecab.h"
#include "mmap.h"
#include "darts.h"
#include "freelist.h"
#include "common.h"
#include "param.h"
#include "string_buffer.h"

namespace MeCab {

  class FeatureIndex
  {
  protected:
    std::vector <int>   feature_;
    ChunkFreeList<int>  feature_freelist_;
    ChunkFreeList<char> char_freelist_;
    std::vector<char*>  unigram_templs_;
    std::vector<char*>  bigram_templs_;
    StringBuffer os_;
    size_t      maxid_;
    double      *alpha_;
    StringBuffer what_;
    virtual int id (const char *) = 0;

    char* getIndex (char* &, char **, size_t);
    bool openTemplate (Param &);

  public:

    virtual bool open (Param &) = 0;
    virtual void clear  () = 0;
    virtual void close  () = 0;
    virtual bool buildFeature (LearnerPath *) = 0;

    void set_alpha(double *);

    size_t size ()           { return maxid_; }
    bool buildUnigramFeature (LearnerPath *);
    bool buildBigramFeature  (LearnerPath *);
    void calcCost            (LearnerPath *);
    void calcCost            (LearnerNode *);

    const char *what() { return what_.str(); }

    char *strdup   (const char *);

    FeatureIndex(): feature_freelist_(8192 * 32),
		    char_freelist_(8192 * 32),
		    maxid_(0), alpha_(0) {};
    virtual ~FeatureIndex() {}
  };

  class EncoderFeatureIndex: public FeatureIndex
  {
  private:
    std::map <std::string, std::pair<int, unsigned int> >   dic_;
    std::map <std::string, int *> feature_cache_;
    int id (const char *);
  public:
    bool save    (const char*);
    bool convert (const char*, const char*);
    void shrink  (size_t, std::vector<double>&) ;
    bool open (Param &);
    void clear ();
    bool buildFeature (LearnerPath *);
    void clearcache ();
    void close ();
  };

  class DecoderFeatureIndex: public FeatureIndex
  {
  private:
    Mmap<char>  mmap_;
    Darts::DoubleArray da_;
    int id (const char *);
  public:
    bool open (Param &);
    bool buildFeature (LearnerPath *);
    void clear ();
    void close ();
  };
}

#endif
