/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: freelist.h,v 1.2 2005/12/24 08:24:22 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _MECAB_FREELIST_H
#define _MECAB_FREELIST_H

#include <iostream>
#include <vector>
#include <algorithm>
#include "common.h"

namespace MeCab {

  template <class T>
  class FreeList
  {
  private:
    std::vector <T *> freeList;
    size_t pi_;
    size_t li_;
    size_t size;

  public:
    void free () { li_ = pi_ = 0; }

    T* alloc ()
    {
      if (pi_ == size) { li_++; pi_ = 0; }
      if (li_ == freeList.size()) freeList.push_back(new T [size]);
      return freeList[li_] + (pi_++);
    }

    FreeList (size_t _size): pi_(0), li_(0), size (_size) {}

    virtual ~FreeList ()
    {
      for (li_ = 0; li_ < freeList.size(); li_++)
	delete [] freeList[li_];
    }
  };

  template <class T>
  class ChunkFreeList
  {
  private:
    std::vector < std::pair <size_t, T *> > freelist_;
    size_t pi_;
    size_t li_;
    size_t default_size;

  public:
    void free () { li_ = pi_ = 0; }

    T* alloc (T *src)
    {
      T* n = alloc (1);
      *n = *src;
      return n;
    }

    T* alloc (size_t req = 1)
    {
      while (li_ < freelist_.size ()) {
	if ( (pi_ + req) < freelist_[li_].first ) {
	  T *r = freelist_[li_].second + pi_;
	  pi_ += req;
	  return r;
	}
	li_++;
	pi_ = 0;
      }
      size_t _size = _max (req, default_size);
      freelist_.push_back (std::make_pair (_size, new T [_size]));
      li_ = freelist_.size() - 1;
      pi_ += req;
      return freelist_[li_].second;
    }

    ChunkFreeList (size_t _size): pi_(0), li_(0), default_size (_size) {}

    virtual ~ChunkFreeList ()
    {
      for (li_ = 0; li_ < freelist_.size(); li_++)
	delete [] freelist_[li_].second;
    }
  };
}
#endif
