/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: learner_node.h,v 1.2 2005/12/24 08:24:22 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _MECAB_LEARNER_NODE_H
#define _MECAB_LEARNER_NODE_H

#include "common.h"
#include "mecab.h"
#include <iostream>
#include <cstring>

namespace MeCab {

  template <class T1, class T2> T1 repeat_find_if(T1 b, T1 e, const T2& v, size_t n)
  {
    T1 r = b;
    for (size_t i = 0; i < n; ++i) {
      r = std::find (b, e, v);
      if (r == e) return e;
      b = r + 1;
    }
    return r;
  }

  /* NOTE: first argment:   answer,
     second argment:  system output */
  inline bool node_cmp_eq (const LearnerNode &node1, const LearnerNode &node2,
			   size_t size, size_t unk_size)
  {
    if (node1.length == node2.length && strncmp (node1.surface, node2.surface, node1.length) == 0) {
      char *p1 = node1.feature;
      char *p2 = node2.feature;
      // There is NO case when node1 becomes MECAB_UNK_NODE
      if (node2.stat == MECAB_UNK_NODE) size = unk_size;  // system cannot output other extra information
      char *r1 = repeat_find_if (p1, p1 + std::strlen (p1), ',', size);
      char *r2 = repeat_find_if (p2, p2 + std::strlen (p2), ',', size);
      if ( (size_t)(r1 - p1) == (size_t)(r2 - p2) &&
	   strncmp (p1, p2, (size_t)(r1 - p1)) == 0) {
	return true;
      }
    }

    return false;
  }

  inline bool is_empty (LearnerPath *path)
  {
    return ((! path->rnode->rpath && path->rnode->stat != MECAB_EOS_NODE) ||
	    (! path->lnode->lpath && path->lnode->stat != MECAB_BOS_NODE) );
  }

  inline void calc_expectation (LearnerPath *path, double *expected, double Z)
  {
    if ( is_empty (path) ) return;

    double c = std::exp (path->lnode->alpha + path->cost + path->rnode->beta - Z);

    for (int *f = path->fvector; *f != -1; ++f) expected[*f] += c;

    if (path->rnode->stat != MECAB_EOS_NODE)
      for (int *f = path->rnode->fvector; *f != -1; ++f) expected[*f] += c;
  }

  inline void calc_alpha (LearnerNode *n)
  {
    n->alpha = 0.0;
    for (LearnerPath *path = n->lpath; path; path = path->lnext)
      n->alpha = logsumexp (n->alpha, path->cost + path->lnode->alpha, path == n->lpath);
  }

  inline void calc_beta (LearnerNode *n)
  {
    n->beta = 0.0;
    for (LearnerPath *path = n->rpath; path; path = path->rnext)
      n->beta = logsumexp (n->beta, path->cost + path->rnode->beta, path == n->rpath);
  }
}

#endif
