/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: learner_tagger.h,v 1.2 2005/12/24 08:24:22 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef MECAB_TAGGER_H
#define MECAB_TAGGER_H

#include <vector>

#include "mecab.h"
#include "freelist.h"
#include "feature_index.h"
#include "tokenizer.h"
#include "scoped_ptr.h"

namespace MeCab
{
  class LearnerTagger
  {
  protected:
    LearnerTokenizer    *tokenizer_;
    FreeList<LearnerPath> *path_freelist_;
    FeatureIndex    *feature_index_;
    scoped_string          begin__;
    char *begin_;
    char *end_;
    size_t len_;
    std::vector <LearnerNode *>    beginNodeList_;
    std::vector <LearnerNode *>    endNodeList_;
    StringBuffer what_;

    LearnerNode *lookup (size_t);
    bool  connect  (size_t, LearnerNode *);
    bool  viterbi  ();
    bool  buildLattice  ();
    bool  initList  ();
  public:
    bool   empty ()  { return (len_ == 0); }
    void close () {};
    const char *what () { return what_.str(); }

    LearnerTagger (): tokenizer_(0), path_freelist_(0), feature_index_(0),
		      begin_(0), end_(0), len_(0) {};

    virtual ~LearnerTagger () {};
  };

  class EncoderLearnerTagger: public LearnerTagger {
  private:
    size_t eval_size_;
    size_t unk_eval_size_;
    std::vector <LearnerPath *>  ansPathList_;
  public:
    bool open  (LearnerTokenizer *,
		FreeList<LearnerPath> *,
		FeatureIndex *, size_t, size_t);
    bool read (std::istream &, std::vector<double>&);
    int  eval (size_t&, size_t&, size_t&);
    double gradient (double *);
    EncoderLearnerTagger(): eval_size_ (1024), unk_eval_size_(1024) {}
    ~EncoderLearnerTagger () { close(); }
  };

  class DecoderLearnerTagger: public LearnerTagger {
  private:
    scoped_ptr<LearnerTokenizer>       tokenizer__;
    scoped_ptr<FreeList<LearnerPath> > path_freelist__;
    scoped_ptr<FeatureIndex>         feature_index__;
  public:
    bool open  (Param &);
    bool parse (std::istream &, std::ostream &);
    ~DecoderLearnerTagger () { close(); }
  };
}

#endif
