/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: libmecab.cpp,v 1.10 2006/04/25 17:16:21 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "mecab.h"
#include "strutil.h"
#include "tokenizer.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define LIBMECAB_ID (77718)

std::string errorStr;

struct mecab_t {
  int allocated;
  MeCab::Tagger* ptr;
};

#if defined  (_WIN32) && ! defined (__CYGWIN__)
#include <windows.h>
HINSTANCE DllInstance = 0;

BOOL __stdcall DllMain( HINSTANCE hinst, DWORD dwReason, void* )
{
  if (! DllInstance) DllInstance = hinst;
  return TRUE;
}
#endif

mecab_t* mecab_new (int argc, char **argv)
{
  mecab_t *c = new mecab_t;
  MeCab::Tagger *ptr = MeCab::createTagger(argc, argv);
  if (! c || ! ptr) {
    delete c; 
    errorStr = MeCab::getTaggerError();
    return 0;
  }
  c->ptr = ptr;
  c->allocated = LIBMECAB_ID;
  return c;
}

mecab_t* mecab_new2 (const char *arg)
{
  mecab_t *c = new mecab_t;
  MeCab::Tagger *ptr = MeCab::createTagger(arg);
  if (! c || ! ptr) {
    delete c;      
    errorStr = MeCab::getTaggerError();
    return 0;
  }
  c->ptr = ptr;
  c->allocated = LIBMECAB_ID;
  return c;
}

const char* mecab_strerror (mecab_t *c)
{
  if (! c || ! c->allocated) return const_cast<char *>(errorStr.c_str ());
  return c->ptr->what();
}

void mecab_destroy (mecab_t *c)
{
  if (c && c->allocated) {
    delete c->ptr;
    delete c;
  }
  c = 0;
}

#define MECAB_CHECK_FIRST_ARG(c,t)  \
 if (! (c) || (c)->allocated != LIBMECAB_ID) { \
    errorStr = "first argment seems to be invalid"; \
    return 0; \
 } MeCab::Tagger *(t) = (c)->ptr;

const char* mecab_sparse_tostr (mecab_t *c, const char *str)
{
  MECAB_CHECK_FIRST_ARG(c,t);
  return t->parse (str);
}

const char* mecab_sparse_tostr2 (mecab_t *c, const char *str, size_t len)
{
  MECAB_CHECK_FIRST_ARG(c,t);
  return t->parse (str, len);
}

char* mecab_sparse_tostr3 (mecab_t *c, const char *str, size_t len,
			   char *out, size_t len2)
{
  MECAB_CHECK_FIRST_ARG(c,t);
  return const_cast<char *>(t->parse (str, len, out, len2));
}

mecab_node_t* mecab_sparse_tonode (mecab_t *c, const char *str)
{
  MECAB_CHECK_FIRST_ARG(c,t);
  return reinterpret_cast<mecab_node_t *>(t->parseToNode (str));
}

mecab_node_t* mecab_sparse_tonode2 (mecab_t *c, const char *str, size_t len)
{
  MECAB_CHECK_FIRST_ARG(c,t);
  return reinterpret_cast<mecab_node_t *>(t->parseToNode (str,len));
}

const char* mecab_nbest_sparse_tostr (mecab_t* c, size_t N, const char *str)
{
  MECAB_CHECK_FIRST_ARG(c,t);
  return t->parseNBest (N, str);
}

const char* mecab_nbest_sparse_tostr2 (mecab_t* c, size_t N, const char* str, size_t len)
{
  MECAB_CHECK_FIRST_ARG(c,t);
  return t->parseNBest (N, str, len);
}

char* mecab_nbest_sparse_tostr3 (mecab_t* c, size_t N, const char *str, size_t len,
				 char *out, size_t len2)
{
  MECAB_CHECK_FIRST_ARG(c,t);
  return const_cast<char *>(t->parseNBest (N, str, len, out, len2));
}

int mecab_nbest_init (mecab_t *c, const char *str)
{
  MECAB_CHECK_FIRST_ARG(c,t);
  return t->parseNBestInit (str);
}

int mecab_nbest_init2 (mecab_t *c, const char *str, size_t len)
{
  MECAB_CHECK_FIRST_ARG(c,t);
  return t->parseNBestInit (str, len);
}

const char* mecab_nbest_next_tostr (mecab_t* c)
{
  MECAB_CHECK_FIRST_ARG(c,t);
  return t->next ();
}

char* mecab_nbest_next_tostr2 (mecab_t* c, char *out, size_t len2)
{
  MECAB_CHECK_FIRST_ARG(c,t);
  return const_cast<char *>(t->next (out, len2));
}

mecab_node_t* mecab_nbest_next_tonode (mecab_t* c)
{
  MECAB_CHECK_FIRST_ARG(c,t);
  return reinterpret_cast<mecab_node_t *>(t->nextNode ());
}

const char* mecab_format_node (mecab_t *c, mecab_node_t* n)
{
  MECAB_CHECK_FIRST_ARG(c,t);
  return t->formatNode (n);
}
