/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: nbest_generator.cpp,v 1.2 2005/12/24 08:24:22 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "mecab.h"
#include "nbest_generator.h"
#include <queue>

namespace MeCab {

  bool NBestGenerator::set (Node *node)
  {
    freelist_.free();
    for (; node->next; node = node->next) {}; // seek to EOS;
    while (! agenda_.empty()) agenda_.pop (); // make empty
    QueueElement *eos = freelist_.alloc ();
    eos->node = node;
    eos->next = 0;
    eos->fx = eos->gx = 0;
    agenda_.push (eos);
    return true;
  }

  Node* NBestGenerator::next ()
  {
    while (! agenda_.empty()) {
      QueueElement *top = agenda_.top ();
      agenda_.pop ();
      Node *rnode = top->node;

      if (rnode->stat == MECAB_BOS_NODE) { // BOS
	for (QueueElement *n = top; n->next; n = n->next) {
	  n->node->next = n->next->node; // change next & prev
	  n->next->node->prev = n->node;
	  // TODO: rewrite costs;
	}
	return rnode;
      }

      for (Path *path = rnode->lpath; path; path = path->lnext) {
	QueueElement *n = freelist_.alloc ();
	n->node = path->lnode;
	n->gx = path->cost + top->gx;
	n->fx = path->lnode->cost + path->cost + top->gx;
	n->next = top;
	agenda_.push (n);
      }
    }
    return 0;
  }
}

