/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: param.h,v 1.2 2005/12/24 08:24:22 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _MECAB_PARAM_H
#define _MECAB_PARAM_H

#include <map>
#include <string>
#include <vector>
#include <iostream>
#include "string_buffer.h"

using namespace std;

namespace MeCab {

  struct Option
  {
    const char *name;
    char        short_name;
    const char *default_value;
    const char *arg_description;
    const char *description;
  };

  class Param
  {
  private:
    std::map    <std::string, std::string> conf_;
    std::vector <std::string> rest_;
    std::string systemName_;
    std::string help_;
    std::string version_;
    StringBuffer what_;

  public:
    bool open    (int,  char**, const Option *);
    bool open    (const char*,  const Option *);
    bool load    (const char*);
    void clear   ();
    const std::vector <std::string>& getRestArg () { return rest_; };

    const char* getSystemName () { return systemName_.c_str(); };
    const char *what ()          { return what_.str(); };

    const char* help    (const Option *);
    const char* version (const Option *);

    const std::string getProfileString (const char*);
    int               getProfileInt    (const char*);
    double            getProfileFloat  (const char*);
    void              setProfile       (const char*, const char* value, bool = true);
    void              setProfile       (const char*, int, bool = true);

    Param ()  {};
    virtual ~Param () {};
  };
}

#endif
