/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: stream_wrapper.h,v 1.2 2005/12/24 08:24:22 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _MECAB_STREAM_WRAPPER_H
#define _MECAB_STREAM_WRAPPER_H

#include <iostream>
#include <fstream>
#include <cstring>

namespace MeCab {

  class istream_wrapper {
  private:
    std::istream* is_;
  public:
    std::istream &operator*() const  { return *is_; }
    std::istream *operator->() const { return is_;  }
    explicit istream_wrapper(const char* filename): is_(0)
    {
      if (std::strcmp(filename, "-") == 0)
	is_ = &std::cin;
      else
	is_ = new std::ifstream(filename);
    }

    virtual ~istream_wrapper()
    {
      if (is_ != &std::cin) delete is_;
    }
  };

  class ostream_wrapper {
  private:
    std::ostream* os_;
  public:
    std::ostream &operator*() const  { return *os_; }
    std::ostream *operator->() const { return os_;  }
    explicit ostream_wrapper(const char* filename): os_(0)
    {
      if (std::strcmp(filename, "-") == 0)
	os_ = &std::cout;
      else
	os_ = new std::ofstream(filename);
    }

    virtual ~ostream_wrapper()
    {
      if (os_ != &std::cout) delete os_;
    }
  };
}

#endif
