/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: string_buffer.h,v 1.3 2005/12/24 08:24:22 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _MECAB_STRINGBUFFER_H
#define _MECAB_STRINGBUFFER_H

#include <string>
#include "common.h"
#include "strutil.h"

namespace MeCab {

#define _ITOA(_n)  char fbuf[64]; itoa  (_n, fbuf);  return this->write (fbuf);
#define _UITOA(_n) char fbuf[64]; uitoa (_n, fbuf);  return this->write (fbuf);
#define _DTOA(_n)  char fbuf[64]; dtoa  (_n, fbuf);  return this->write (fbuf);

  class StringBuffer
  {
  private:
    size_t size_;
    size_t alloc_size_;
    char *ptr_;
    bool is_delete_;
    bool error_;
    bool reserve (size_t);

  public:
    explicit StringBuffer (): size_(0), alloc_size_(0),
			      ptr_(0), is_delete_ (true), error_ (false) {}
    explicit StringBuffer (char *_s, size_t _l):
      size_(0), alloc_size_(_l), ptr_ (_s), is_delete_(false), error_(false) {}

    virtual ~StringBuffer ();

    StringBuffer& write (char);
    StringBuffer& write (const char*, size_t);
    StringBuffer& write (const char*);

    StringBuffer& operator<< (double _n)             { _DTOA(_n); }
    StringBuffer& operator<< (short int _n)          { _ITOA(_n); }
    StringBuffer& operator<< (int _n)                { _ITOA(_n); }
    StringBuffer& operator<< (long int _n)           { _ITOA(_n); }
    StringBuffer& operator<< (unsigned short int _n) { _UITOA(_n); }
    StringBuffer& operator<< (unsigned int _n)       { _UITOA(_n); }
    StringBuffer& operator<< (unsigned long int _n)  { _UITOA(_n); }
    StringBuffer& operator<< (char _n)               { return this->write (_n); }
    StringBuffer& operator<< (unsigned char _n)      { return this->write (_n); }
    StringBuffer& operator<< (const char* _n)        { return this->write (_n); }
    StringBuffer& operator<< (const std::string& _n) { return this->write (_n.c_str()); }

    void clear () { size_ = 0; }
    const char *str () { return error_ ?  0 : const_cast<const char*>(ptr_); }
  };
}

#endif
