/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: tokenizer.h,v 1.5 2006/01/10 06:30:00 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _MECAB_TOKENIZER_H
#define _MECAB_TOKENIZER_H

#include "mecab.h"
#include "freelist.h"
#include "param.h"
#include "dictionary.h"
#include "char_property.h"
#include "scoped_ptr.h"
#include "string_buffer.h"
#include "token.h"

namespace MeCab {

  template <typename __node_t, typename __path_t>
  class TokenizerImpl
  {
  private:
    std::vector <Dictionary *> dic_;
    Dictionary unkdic_;
    scoped_string bos_feature_;
    FreeList<__node_t> node_freelist_;
    std::vector<std::pair<Token *, size_t> > unk_tokens_;
    scoped_array<Dictionary::result_type> daresults_;
    CharInfo space_;
    CharProperty property_;
    unsigned int id_;
    StringBuffer what_;

  public:

    __node_t *getNewNode ()
    {
      __node_t *node = node_freelist_.alloc ();
      std::memset (node, 0, sizeof (__node_t));
      node->id = id_++;
      return node;
    }

    __node_t *getBOSNode ();
    __node_t *getEOSNode ();

    void preprocess   (const char *, const char *) {};
    void postprocess  (const char *, const char *) {};
    __node_t *lookup  (const char *, const char *);

    bool open (Param &);
    void close ();
    void clear ();

    const char *what () {  return what_.str(); }

    TokenizerImpl ();
    ~TokenizerImpl () { this->close (); }
  };

  typedef TokenizerImpl<Node, Path>               Tokenizer;
  typedef TokenizerImpl<LearnerNode, LearnerPath> LearnerTokenizer;
}
#endif
