/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: viterbi.h,v 1.7 2006/03/17 16:56:34 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _MECAB_VITERBI_H
#define _MECAB_VITERBI_H

#include <vector>
#include <iostream>
#include "mecab.h"
#include "tokenizer.h"
#include "connector.h"
#include "freelist.h"
#include "param.h"
#include "string_buffer.h"

namespace MeCab {

  class Viterbi
  {
  private:
    Tokenizer* tokenizer_;
    Connector *connector_;
    Node*      eosNode_;
    Node*      bosNode_;
    char*      begin_;
    char*      end_;
    int        level_;
    double     theta_;
    double     Z_; 
    bool copy_sentence_; 
    bool partial_;
    scoped_ptr<FreeList<Path> > path_freelist_;
    std::vector<Node *> end_node_list_;
    std::vector<Node *> begin_node_list_;
    std::vector<char> sentence_;
    std::vector<char> constraint_buf_;

    bool  (Viterbi::*analyze_)  (const char *,size_t);
    bool  (Viterbi::*connect_)   (size_t, Node*);
    Node *(Viterbi::*buildLattice_) ();

    StringBuffer what_;

    Node *lookup                (size_t);

    bool connectNormal          (size_t, Node*);
    bool connectWithAllPath     (size_t, Node*);

    bool forwardbackward       (const char*, size_t);
    bool viterbi               (const char*, size_t);

    Node *filterNode     (Node *, size_t);
    bool initConstraints (const char*&, size_t&);

    Node *buildBestLattice();
    Node *buildAllLattice();

    inline bool connect (size_t p, Node* n) {
      return (this->*connect_) (p, n); 
    }

  public:
    void   clear ();
    bool   open (Param&, Tokenizer*, Connector *);
    void   close ();
     
    double Z () const  { return Z_; }

    Node *analyze (const char *str, size_t len);

    const char *what () { return what_.str(); }

    explicit Viterbi ();
    virtual ~Viterbi ();
  };
}
#endif
