/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: viterbisub.h,v 1.3 2006/04/25 17:16:21 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "viterbi.h"

// for MSVC
#undef max

namespace MeCab {

#ifdef _VITERBI_WITH_ALL_PATH
  bool Viterbi::connectWithAllPath
#else
  bool Viterbi::connectNormal
#endif
  (size_t pos, Node *rNode)
  {
    for (;rNode; rNode = rNode->bnext) {

      register long bestCost = 2147483647;

      Node* bestNode = 0;

      for (Node *lNode = end_node_list_[pos]; lNode; lNode = lNode->enext) {

#ifdef _VITERBI_WITH_ALL_PATH
	register int lcost = connector_->getCost (lNode, rNode); // local cost
	register long cost  = lNode->cost + lcost;
#else
	register long cost  = lNode->cost + connector_->getCost (lNode, rNode);
#endif

	if (cost < bestCost) {
	  bestNode  = lNode;
	  bestCost  = cost;
	}

#ifdef _VITERBI_WITH_ALL_PATH
	Path *path   = path_freelist_->alloc ();
	path->cost   = lcost;
	path->rnode  = rNode;
	path->lnode  = lNode;
	path->lnext  = rNode->lpath;
	rNode->lpath = path;
	path->rnext  = lNode->rpath;
	lNode->rpath = path;
#endif
      }

      // overflow check 2003/03/09
      if (! bestNode) {
	WHAT("too long sentence.");
	return false;
      }

      rNode->prev = bestNode;
      rNode->next = 0;
      rNode->cost = bestCost;
      size_t x    = rNode->rlength + pos;
      rNode->enext = end_node_list_[x];
      end_node_list_[x] = rNode;
    }

    return true;
  }
}
