/*
  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer

  $Id: writer.h,v 1.3 2005/12/24 08:24:22 taku-ku Exp $;

  Copyright (C) 2001-2005 Taku Kudo <taku@chasen.org>
  Copyright (C) 2004-2005 Nippon Telegraph and Telephone Corporation

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string>
#include "mecab.h"
#include "scoped_ptr.h"
#include "string_buffer.h"

namespace MeCab {

  class Param;

  class Writer
  {
  private:
    scoped_string  node_format_;
    scoped_string  bos_format_;
    scoped_string  eos_format_;
    scoped_string  unk_format_;
    StringBuffer what_;

    bool writeNode         (StringBuffer&, const char*, const char*, Node*);
    bool writeLattice      (StringBuffer&, const char*, Node*);
    bool writeWakati       (StringBuffer&, const char*, Node*);
    bool writeNone         (StringBuffer&, const char*, Node*);
    bool writeUser         (StringBuffer&, const char*, Node*);
    bool writeDump         (StringBuffer&, const char*, Node*);
    bool (Writer::*_write) (StringBuffer&, const char*, Node*);

  public:
    Writer ();
    virtual ~Writer();
    bool open (Param &);
    void close ();
    bool writeNode (StringBuffer &, const char*, Node *);
    bool write     (StringBuffer &, const char*, Node *);
    const char *what () { return what_.str(); }
  };
}

